IMPLEMENTATION MODULE DLISTS;

FROM SYSTEM IMPORT SIZE;

FROM Storage
   IMPORT Available, ALLOCATE, DEALLOCATE;

FROM ERRORHANDLER
   IMPORT ErrorHandler;

FROM DLISTSEXCEPTIONS IMPORT Exit;

IMPORT ITEMS;

CONST ModuleID = "DLISTS";

TYPE List = POINTER TO Node;
     Node = RECORD
        prev : List;
        Data : ITEMS.Item;
        next : List
     END (* Node *);



   PROCEDURE Create () : List;
   BEGIN
      RETURN NullList
   END Create;


   PROCEDURE Destroy (VAR L : List);
   BEGIN
      WHILE L # NullList DO
         Clear (L)
      END (* WHILE *)
   END Destroy;


   PROCEDURE Clear (VAR L : List);
   CONST ProcID = "Clear";
   VAR node : List;
   BEGIN
      IF L # NullList THEN
         node := L;
         L := L^.next;
         ITEMS.Destroy (node^.Data);
         DEALLOCATE (node,SIZE (Node))
      ELSE
         ErrorHandler (ModuleID,ProcID,1,Exit)
      END (* IF *)
   END Clear;


      PROCEDURE append (    node        : List;
                        VAR first, last : List);
      BEGIN
         IF first = NullList THEN
            first := node
         ELSE
            last^.next := node
         END (* IF *);
         node^.prev := last;
         last := node
      END append;


   PROCEDURE Copy (    Source : List;
                   VAR Target : List);
   CONST ProcID = "Copy";
   VAR newNode,
       last      : List;
   BEGIN
      IF Source # Target THEN
         Clear (Target);
         last := NullList;
         WHILE Source # NullList DO
            IF Available (SIZE (Node)) THEN
               ALLOCATE (newNode,SIZE (Node))
            ELSE
               ErrorHandler (ModuleID,ProcID,2,Exit)
            END (* IF *);
            WITH newNode^ DO
               Data := Source^.Data;
               next := NullList
            END (* WITH *);
            append (newNode,Target,last);
            Source := Source^.next
         END (* WHILE *)
      END (* IF *)
   END Copy;


   PROCEDURE Insert (VAR L : List;
                         I : ITEMS.Item);
   CONST ProcID = "Insert";
   VAR newList : List;
   BEGIN
      IF Available (SIZE (Node)) THEN
         NEW (newList);
         WITH newList^ DO
            prev := L^.prev;
            Data := I;
            next := L
         END (* WITH *);
         IF L^.prev # NullList THEN
            L^.prev^.next := newList
         END (* IF *);
         L^.prev := newList;
         L := newList
      ELSE
         ErrorHandler (ModuleID,ProcID,2,Exit)
      END (* IF *)
   END Insert;


   PROCEDURE SetList (I : ITEMS.Item) : List;
   VAR newList : List;
   CONST ProcID = "SetList";
   BEGIN
      IF Available (SIZE (Node)) THEN
         ALLOCATE (newList,SIZE (Node));
         WITH newList^ DO
            prev := NullList;
            Data := I;
            next := NullList
         END (* WITH *);
      ELSE
         ErrorHandler (ModuleID,ProcID,2,Exit)
      END (* IF *);
      RETURN newList
   END SetList;


   PROCEDURE SetItem (    I : ITEMS.Item;
                      VAR L : List);
   CONST ProcID = "SetItem";
   BEGIN
      IF L = NullList THEN
         ErrorHandler (ModuleID,ProcID,3,Exit)
      ELSE
         L^.Data := I
      END (* IF *)
   END SetItem;


   PROCEDURE SetNext (VAR L : List;
                          N : List);
   CONST ProcID = "SetNext";
   BEGIN
      IF L = NullList THEN
         ErrorHandler (ModuleID,ProcID,4,Exit)
      ELSE
         L^.next := N
      END (* IF *)
   END SetNext;


   PROCEDURE SetPrev (VAR L : List;
                          P : List);
   CONST ProcID = "SetPrev";
   BEGIN
      IF L = NullList THEN
         ErrorHandler (ModuleID,ProcID,5,Exit)
      ELSE
         L^.prev := P
      END (* IF *)
   END SetPrev;


   PROCEDURE IsEmpty (L : List) : BOOLEAN;
   BEGIN
      RETURN L = NullList
   END IsEmpty;


   PROCEDURE IsEqual (left  : List;
                      right : List) : BOOLEAN;
   BEGIN
      WHILE (left # NullList) & (right # NullList) DO
         IF left^.Data # right^.Data THEN
            RETURN FALSE
         END (* IF *);
         left := left^.next;
         right := right^.next
      END (* WHILE *);
      RETURN (left = right) & (right = NullList)
   END IsEqual;


   PROCEDURE LengthOf (L : List) : CARDINAL;
   VAR length : CARDINAL;
   BEGIN
      length := 0;
      WHILE L # NullList DO
         INC (length);
         L := L^.next
      END (* WHILE *);
      RETURN length
   END LengthOf;


   PROCEDURE GetItem (L : List) : ITEMS.Item;
   BEGIN
      IF L = NullList THEN
         RETURN ITEMS.NullItem
      END (* IF *);
      RETURN L^.Data
   END GetItem;


   PROCEDURE GetNext (L : List) : List;
   BEGIN
      IF L = NullList THEN
         RETURN NullList
      END (* IF *);
      RETURN L^.next
   END GetNext;


   PROCEDURE GetPrev (L : List) : List;
   BEGIN
      IF L = NullList THEN
         RETURN NullList
      END (* IF *);
      RETURN L^.prev
   END GetPrev;

BEGIN (************* Module Initialization *****************)
   NullList := NIL
END DLISTS.
