IMPLEMENTATION MODULE ITEMS;

FROM ERRORHANDLER IMPORT ErrorHandler;

FROM ITEMSEXCEPTIONS IMPORT Exit;

FROM SYSTEM IMPORT BYTE, SIZE, ADDRESS, ADR;

FROM Storage IMPORT ALLOCATE, DEALLOCATE, Available;

FROM System IMPORT Move;


CONST ModuleID = "ITEMS";


TYPE Item = POINTER TO ItemRecord;
     ItemRecord = RECORD
        Size : CARDINAL;
        Data : ADDRESS
     END (* ItemRecord *);

   PROCEDURE Create (Data : ARRAY OF BYTE) : Item;
   CONST ProcID = "Create";
   VAR newItem : Item;
   BEGIN
      IF Available (SIZE(ItemRecord)) THEN
         ALLOCATE (newItem, SIZE(ItemRecord));
         newItem^.Size := SIZE (Data);
         IF Available (newItem^.Size) THEN
            ALLOCATE (newItem^.Data,newItem^.Size);
            Move (ADR (Data), newItem^.Data, newItem^.Size)
         ELSE
            ErrorHandler (ModuleID,ProcID,1,Exit);
            newItem^.Size := 0;
            newItem^.Data := NullItem
         END (* IF *);
      ELSE
         ErrorHandler (ModuleID,ProcID,2,Exit);
         newItem := NullItem
      END (* IF *);
      RETURN newItem
   END Create;


   PROCEDURE Clear (VAR I : Item);
   CONST ProcID = "Clear";
   BEGIN
      IF IsDefined (I) THEN
         DEALLOCATE (I^.Data,I^.Size);
         I^.Size := 0;
         I^.Data := NullItem
      ELSE
         ErrorHandler (ModuleID,ProcID,3,Exit);
      END (* IF *)
   END Clear;


   PROCEDURE Destroy (VAR I : Item);
   BEGIN
      IF IsDefined (I) THEN
         IF NOT IsNull (I) THEN
            Clear (I)
         END (* IF *);
         DEALLOCATE (I,SIZE (ItemRecord));
         I := NullItem
      END (* IF *)
   END Destroy;


   PROCEDURE GetItem (VAR Data : ARRAY OF BYTE;
                          I    : Item);
   CONST ProcID = "GetItem";
   BEGIN
      IF IsDefined (I) THEN
         IF NOT IsNull (I) THEN
            Move (I^.Data, ADR (Data), I^.Size)
         ELSE
            ErrorHandler (ModuleID,ProcID,4,Exit);
         END (* IF *)
      ELSE
         ErrorHandler (ModuleID,ProcID,5,Exit);
      END (* IF *)
   END GetItem;


   PROCEDURE SetItem (    Data : ARRAY OF BYTE;
                      VAR I    : Item);
   BEGIN
      IF NOT IsDefined (I) THEN
         I := Create (Data)
      ELSE
         I^.Size := SIZE (Data);
         Move (I^.Data,ADR (Data),I^.Size)
      END (* IF *)
   END SetItem;


   PROCEDURE IsDefined (I : Item) : BOOLEAN;
   BEGIN
      RETURN I # NullItem
   END IsDefined;


   PROCEDURE IsNull (I : Item) : BOOLEAN;
   BEGIN
      RETURN I^.Data = NullItem
   END IsNull;


BEGIN (******** Module Initialization ************)
   NullItem := NIL
END ITEMS.
